﻿using HELLo_World.MapData.People;
using System;
using System.Collections.Generic;
using System.Text;

namespace HELLo_World.MapData.Buildings
{
    public abstract class Facility : Building
    {
        #region Properties

        public Int32 Fee { get; set; }

        public FacilityStatus Status { get; set; }

        public Int32 TimeSpentInStatus { get; set; }

        public Queue<Visitor> UserList { get; set; }

        public Queue<Visitor> VisitorQueue { get; set; }

        #endregion

        #region Constructor

        public Facility(Int32 bId, Int32 tId, Tuple<Int32, Int32> coord, Int32 startingFee) : base(bId, tId, coord)
        {
            Fee = startingFee;
            Status = FacilityStatus.UNDER_CONSTRUCTION;
            TimeSpentInStatus = 0;
            UserList = new Queue<Visitor>();
            VisitorQueue = new Queue<Visitor>();
        }

        #endregion

        #region Public methods

        public void Leave(Visitor visitor)
        {
            int u = UserList.Count;
            int w = VisitorQueue.Count;
            Queue<Visitor> userList = new Queue<Visitor>();
            Queue<Visitor> waitingList = new Queue<Visitor>();

            for (int i = 0; i < u; ++i)
            {
                Visitor ghost = UserList.Dequeue();
                if (visitor.PersonId != ghost.PersonId)
                {
                    userList.Enqueue(ghost);
                }
            }

            for (int i = 0; i < w; ++i)
            {
                Visitor ghost = VisitorQueue.Dequeue();
                if (visitor.PersonId != ghost.PersonId)
                {
                    waitingList.Enqueue(ghost);
                }
            }

            UserList = userList;
            VisitorQueue = waitingList;
        }

        public void Empty()
        {
            UserList.Clear();
            VisitorQueue.Clear();
        }

        #endregion
    }
}
