﻿using HELLo_World.MapData.Buildings;
using System;
using System.Collections.Generic;
using System.Text;
using static System.Math;

namespace HELLo_World.MapData.People
{
    public abstract class Person
    {
        #region Constants

        private static readonly Int32 _pixerPerStep = 5;

        #endregion

        #region Private Variables

        private Tuple<Int32, Int32> _nextTile;
        private Boolean _toCenter;

        #endregion

        #region Properties

        public Int32 PersonId { get; private set; }

        public Tuple<Int32,Int32> Coord { get; set; }

        public Tuple<Int32,Int32> InTileCoord { get; set; }

        public Stack<Tuple<Int32, Int32>> Path { get; set; }

        public Activity CurrentActivity { get; set; }

        public Facility CurrentBuilding { get; set; }

        #endregion

        #region Constructor

        public Person(Int32 id, Tuple<Int32,Int32> coord, Tuple<Int32,Int32> tileCoord)
        {
            PersonId = id;
            Coord = coord;
            InTileCoord = tileCoord;
            CurrentActivity = Activity.WANDERING;
            CurrentBuilding = null;
            _nextTile = null;
            _toCenter = false;
        }

        #endregion

        #region Public Methods

        public void Step()
        {
            if(Path is null) return;
            if (Path.Count == 0 && Coord.Equals(_nextTile) && InTileCoord.Item1 == 25 && InTileCoord.Item2 == 25)
            {
                switch(CurrentActivity)
                {
                    case Activity.MOVING:
                        CurrentActivity = Activity.WAITING;
                        break;
                    case Activity.WANDERING:
                        CurrentActivity = Activity.ARRIVED_TO_ROAD;
                        break;
                    case Activity.LEAVING:
                        CurrentActivity = Activity.ARRIVED_TO_EXIT;
                        break;
                    case Activity.GOING_TO_REST:
                        CurrentActivity = Activity.RESTING;
                        break;
                }
            }

            if ((Coord.Equals(_nextTile) || _nextTile is null) && Path.Count > 0)
            {
                _nextTile = Path.Pop();
            }

            Int32 yDirection, xDirection;

            if (InTileCoord.Item1 == 25 && InTileCoord.Item2 == 25) _toCenter = false;

            if (_toCenter)
            {
                yDirection = InTileCoord.Item1 == 25 ? 0 : (25 - InTileCoord.Item1) / Math.Abs(25 - InTileCoord.Item1) * _pixerPerStep;
                xDirection = InTileCoord.Item2 == 25 ? 0 : (25 - InTileCoord.Item2) / Math.Abs(25 - InTileCoord.Item2) * _pixerPerStep;
            }
            else
            {
                yDirection = (_nextTile.Item1 - Coord.Item1) * _pixerPerStep;
                xDirection = (_nextTile.Item2 - Coord.Item2) * _pixerPerStep;
            }

            InTileCoord = new Tuple<int, int>(InTileCoord.Item1 + yDirection, InTileCoord.Item2 + xDirection);

            Int32 yInTile = InTileCoord.Item1 >= 50 ? 1 : (InTileCoord.Item1 <= 0 ? -1 : 0);
            Int32 xInTile = InTileCoord.Item2 >= 50 ? 1 : (InTileCoord.Item2 <= 0 ? -1 : 0);

            if (yInTile != 0 || xInTile != 0)
            {
                Coord = new Tuple<int, int>(yInTile + Coord.Item1, xInTile + Coord.Item2);
                yInTile = (-50 * yInTile + 50)/2;
                xInTile = (-50 * xInTile + 50)/2;
                InTileCoord = new Tuple<int, int>(yInTile, xInTile);
                _toCenter = true;
            }
        }

        #endregion
    }
}
