﻿using HELLo_World.MapData;
using HELLo_World.MapData.Buildings;
using System;
using System.IO;
using System.Threading.Tasks;


namespace HELLo_World.Persistence.Text
{
    public class TextFileGamePersistence : IGamePersistence
    {
        public async Task<GameMap> LoadAsync(string path)
        {
            GameMap gameMap = null;

            try
            {
                using (StreamReader reader = new StreamReader(path))
                {
                    String line = await reader.ReadLineAsync();
                    gameMap = new GameMap(line);

                    line = await reader.ReadLineAsync();
                    String[] numbers = line.Split(' ');
                    gameMap.Money = Int32.Parse(numbers[0]);
                    gameMap.Ticket = Int32.Parse(numbers[1]);
                    gameMap.Time = Int32.Parse(numbers[2]);
                    gameMap.Buildings = new System.Collections.Generic.List<Building>();

                    // Buildings
                    await reader.ReadLineAsync();
                    line = await reader.ReadLineAsync();
                    numbers = line.Split(' ');
                    Int32 numberOfBuildings = Int32.Parse(numbers[0]);

                    for (Int32 i = 0; i < numberOfBuildings; ++i)
                    {
                        line = await reader.ReadLineAsync();
                        numbers = line.Split(' ');
                        if (Int32.Parse(numbers[1]) == -1)
                        {
                            gameMap.Buildings.Add(new Gate(Int32.Parse(numbers[0]), Int32.Parse(numbers[1]), new Tuple<int, int>(Int32.Parse(numbers[2]), Int32.Parse(numbers[3]))));
                        }
                        else if (Int32.Parse(numbers[1]) == 0)
                        {
                            gameMap.Buildings.Add(new Road(Int32.Parse(numbers[0]), Int32.Parse(numbers[1]), new Tuple<int, int>(Int32.Parse(numbers[2]), Int32.Parse(numbers[3]))));
                        }
                        else
                        {
                            String line2 = await reader.ReadLineAsync();
                            String[] numbers2 = line2.Split(' ');
                            if (Int32.Parse(numbers[1]) < 100)      // Catering
                            {
                                CateringFacility c =  new CateringFacility(
                                    Int32.Parse(numbers[0]), Int32.Parse(numbers[1]), new Tuple<int, int>(Int32.Parse(numbers[2]), Int32.Parse(numbers[3])),
                                    Int32.Parse(numbers2[0]));
                                c.Status = (FacilityStatus) Enum.Parse(typeof(FacilityStatus), numbers2[1]);
                                if (c.Status == FacilityStatus.OPERATING) c.Status = FacilityStatus.WAITING;
                                if (c.Status == FacilityStatus.REPAIRING) c.Status = FacilityStatus.FAULTY;
                                c.TimeSpentInStatus = Int32.Parse(numbers2[2]);
                                gameMap.Buildings.Add(c);
                            } 
                            else if (Int32.Parse(numbers[1]) >= 100) // Entertainment
                            {
                                String line3 = await reader.ReadLineAsync();
                                String[] numbers3 = line3.Split(' ');
                                EntertainmentFacility e = new EntertainmentFacility(
                                    Int32.Parse(numbers[0]), Int32.Parse(numbers[1]), new Tuple<int, int>(Int32.Parse(numbers[2]), Int32.Parse(numbers[3])),
                                    Int32.Parse(numbers2[0]), 
                                    Int32.Parse(numbers3[0]));
                                e.Status = (FacilityStatus)Enum.Parse(typeof(FacilityStatus), numbers2[1]);
                                if (e.Status == FacilityStatus.OPERATING) e.Status = FacilityStatus.WAITING;
                                if (e.Status == FacilityStatus.REPAIRING) e.Status = FacilityStatus.FAULTY;
                                e.TimeSpentInStatus = Int32.Parse(numbers2[2]);
                                gameMap.Buildings.Add(e);
                            }
                        }
                    }
                }
            }
            catch
            {
                throw new GameDataException("Unable to load game");
            }

            return gameMap;
        }

        public async Task SaveAsync(string path, GameMap gameMap)
        {
            try
            {
                using (StreamWriter writer = new StreamWriter(path))
                {
                    await writer.WriteLineAsync(gameMap.Name);
                    await writer.WriteLineAsync(gameMap.Money + " " + gameMap.Ticket + " " + gameMap.Time);

                    await writer.WriteLineAsync("// Buildings");
                    await writer.WriteLineAsync(gameMap.Buildings.Count.ToString());
                    foreach (Building building in gameMap.Buildings)
                    {
                        await writer.WriteLineAsync(building.BuildingId + " " + building.BuildingTypeId + " " + building.Coord.Item1 + " " + building.Coord.Item2);
                        if (building is Facility)
                        {
                            Facility facility = building as Facility;
                            await writer.WriteLineAsync(facility.Fee + " " + facility.Status + " " + facility.TimeSpentInStatus);
                            if (facility is EntertainmentFacility)
                            {
                                EntertainmentFacility entertainmentFacility = facility as EntertainmentFacility;
                                await writer.WriteLineAsync(entertainmentFacility.MinimumCapacity.ToString());
                            }
                        }
                    }
                }
            }
            catch
            {
                throw new GameDataException("Unable to save game");
            }
        }
    }
}
