using System;
using System.Windows.Input;
using System.Runtime.CompilerServices;
using System.Collections.ObjectModel;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Windows.Threading;

using HELLo_World.Model.StaticData;
using HELLo_World.Model;
using HELLo_World.Model.Information;
using HELLo_World.Model.EventArguments;
using HELLo_World.MapData.Buildings;
using HELLo_World.MapData.People;
using System.Linq;
using System.Windows.Data;
using System.Collections.Generic;
using System.Windows.Controls;
using HELLo_World.ViewModel.Fields;

namespace HELLo_World.ViewModel.ViewModels
{
    public enum GameMode { NORMAL, DESTROY, BUILD }

    public class GameViewModel : ViewModelBase
    {
        #region Private variables

        private GameModel _model;
        private String _parkName;
        private Int32 _parkMoney;
        private Int32 _parkGuests;
        private Int32 _income;
        private String _time;
        private Int32 _mapSize;
        private Boolean _buildingInfoPopup;
        private Int32 _buildingInfoX;
        private Int32 _buildingInfoY;
        private String _buildingPopupName;
        private FacilityStatus _buildingPopupStatus;
        private Int32 _buildingPopupFee;
        private Int32 _buildingMinimumCapacity;
        private Int32 _buildingAdrenalin;
        private Tuple<Tuple<Int32, Int32>, Int32> _buildingPopupCapacity;
        private Boolean _visitorInfoPopup;
        private Int32 _visitorInfoX;
        private Int32 _visitorInfoY;
        private MapData.People.Activity _visitorActivity;
        private Status _visitorStatus;
        private Int32 _visitorMoney;
        private Int32 _visitorAdrenalin;
        private Int32 _visitorFood;
        private Int32 _visitorHappiness;
        private Boolean _errorPopup;
        private Int32 _errorX;
        private Int32 _errorY;
        private String _errorMessage;
        private Double _zoom;
        private Speed _lastSpeed;
        private String _speedText;
        private String _pauseText;
        private String _openParkText;
        private FacilityField _selectedBuilding;

        #endregion

        #region UI style properties + private variables

        private Brush _buildBackgound;
        private Brush _destroyBackgound;
        private System.Windows.Visibility _destroyVisibility;
        private System.Windows.Visibility _timeVisibility;
        private System.Windows.Visibility _openVisibility;
        private System.Windows.Visibility _minimumCapacityVisibility;
        private System.Windows.Visibility _buildingAdrenalinVisibility;

        public Int32 PanelWidth { get; private set; }
        public Int32 CanvasHeight { get; private set; }
        public Int32 CanvasWidth { get; private set; }
        public Int32 WindowHeight { get; private set; }
        public Int32 MaxHeight { get { return WindowHeight - 38; } }
        public Int32 WindowWidth { get; private set; }
        public Uri BackgroundImage { get; private set; }
        public Brush DestroyBackground
        {
            get { return _destroyBackgound; }
            private set
            {
                if (_destroyBackgound == value) return;
                _destroyBackgound = value;
                OnPropertyChanged();
            }
        }
        public Brush BuildBackground
        {
            get { return _buildBackgound; }
            private set
            {
                if (_buildBackgound == value) return;
                _buildBackgound = value;
                OnPropertyChanged();
            }
        }

        public System.Windows.Visibility DestroyVisibility
        {
            get { return _destroyVisibility; }
            set 
            {
                if (_destroyVisibility == value) return;
                _destroyVisibility = value;
                OnPropertyChanged("DestroyVisibility");
            }
        }

        public System.Windows.Visibility TimeVisibility
        {
            get { return _timeVisibility; }
            set
            {
                if (_timeVisibility == value) return;
                _timeVisibility = value;
                OnPropertyChanged("TimeVisibility");
            }
        }

        public System.Windows.Visibility OpenVisibility
        {
            get { return _openVisibility; }
            set
            {
                if (_openVisibility == value) return;
                _openVisibility = value;
                OnPropertyChanged("OpenVisibility");
            }
        }

        public System.Windows.Visibility MinimumCapacityVisibility
        {
            get { return _minimumCapacityVisibility; }
            set
            {
                if (_minimumCapacityVisibility == value) return;
                _minimumCapacityVisibility = value;
                OnPropertyChanged("MinimumCapacityVisibility");
            }
        }

        public System.Windows.Visibility BuildingAdrenalinVisibility
        {
            get { return _buildingAdrenalinVisibility; }
            set
            {
                if (_buildingAdrenalinVisibility == value) return;
                _buildingAdrenalinVisibility = value;
                OnPropertyChanged("BuildingAdrenalinVisibility");
            }
        }

        #endregion

        #region Public properties

        public String SpeedText
        {
            get { return _speedText; }
            set
            {
                if (_speedText == value) return;
                _speedText = value;
                OnPropertyChanged("SpeedText");
            }
        }

        public String PauseText
        {
            get { return _pauseText; }
            set
            {
                if (_pauseText == value) return;
                _pauseText = value;
                OnPropertyChanged("PauseText");
            }
        }

        public String OpenParkText
        {
            get { return _openParkText; }
            set
            {
                if (_openParkText == value) return;
                _openParkText = value;
                OnPropertyChanged("OpenParkText");
            }
        }

        public MenuField SelectedItem { get; set; }

        public Boolean IsSelected { get; set; }

        public String ErrorMessage
        {
            get { return _errorMessage; }
            set
            {
                if (_errorMessage == value) return;
                _errorMessage = value;
                OnPropertyChanged("ErrorMessage");
            }
        }

        public Int32 ErrorY
        {
            get { return _errorY; }
            set
            {
                if (_errorY == value) return;
                _errorY = value;
                OnPropertyChanged("ErrorY");
            }
        }

        public Int32 ErrorX
        {
            get { return _errorX; }
            set
            {
                if (_errorX == value) return;
                _errorX = value;
                OnPropertyChanged("ErrorX");
            }
        }

        public Boolean ErrorPopup
        {
            get { return _errorPopup; }
            set
            {
                if (_errorPopup == value) return;
                _errorPopup = value;
                OnPropertyChanged("ErrorPopup");
            }
        }

        public FacilityField BuildingPopupSource { get; set; }

        public Boolean BuildingInfoPopup
        {
            get { return _buildingInfoPopup; }
            set
            {
                if (_buildingInfoPopup == value) return;
                _buildingInfoPopup = value;
                OnPropertyChanged("BuildingInfoPopup");
            }
        }

        public Int32 BuildingInfoX
        {
            get { return _buildingInfoX; }
            set
            {
                if (_buildingInfoX == value) return;
                _buildingInfoX = value;
                OnPropertyChanged("BuildingInfoX");
            }
        }

        public Int32 BuildingInfoY
        {
            get { return _buildingInfoY; }
            set
            {
                if (_buildingInfoY == value) return;
                _buildingInfoY = value;
                OnPropertyChanged("BuildingInfoY");
            }
        }

        public String BuildingPopupName
        {
            get { return "Name: " + _buildingPopupName; }
            set
            {
                if (_buildingPopupName == value) return;
                _buildingPopupName = value;
                OnPropertyChanged("BuildingPopupName");
            }
        }

        public FacilityStatus Status
        {
            get { return _buildingPopupStatus; }
            set
            {
                if (_buildingPopupStatus == value) return;
                _buildingPopupStatus = value;

                String status = String.Empty;
                switch (_buildingPopupStatus)
                {
                    case FacilityStatus.OPERATING:
                        status = "Operating";
                        break;
                    case FacilityStatus.WAITING:
                        status = "Waiting";
                        break;
                    case FacilityStatus.UNDER_CONSTRUCTION:
                        status = "Under construction";
                        break;
                    case FacilityStatus.FAULTY:
                        status = "Faulty";
                        break;
                    case FacilityStatus.REPAIRING:
                        status = "Being repaired";
                        break;
                }

                BuildingPopupStatus = "Status: " + status;
                OnPropertyChanged("BuildingPopupStatus");
            }
        }

        public Tuple<Tuple<Int32, Int32>, Int32> Capacity
        {
            get { return _buildingPopupCapacity; }
            set
            {
                if (_buildingPopupCapacity == value) return;
                _buildingPopupCapacity = value;

                BuildingPopupCapacity = "Capacity: " + _buildingPopupCapacity.Item1.Item1.ToString() + "/" + _buildingPopupCapacity.Item2;
                BuildingPopupQueue = "In Queue: " + _buildingPopupCapacity.Item1.Item2.ToString();
                OnPropertyChanged("BuildingPopupCapacity");
                OnPropertyChanged("BuildingPopupQueue");

                BuildingCurrentCapacity = _buildingPopupCapacity.Item1.Item1;
                OnPropertyChanged("BuildingCurrentCapacity");

                BuildingMaxCapacity = _buildingPopupCapacity.Item2;
                OnPropertyChanged("BuildingMaxCapacity");
            }
        }

        public Int32 Fee
        {
            get { return _buildingPopupFee; }
            set
            {
                if (value > 1000 || value < 0) return;
                if (_buildingPopupFee == value) return;
                _buildingPopupFee = value;
                _model.SetBuildingFee(_selectedBuilding.Id, value);

                OnPropertyChanged("Fee");
            }
        }

        public Int32 MinimumCapacity
        {
            get { return _buildingMinimumCapacity; }
            set
            {
                if (value > Capacity.Item2 || value < 0) return;
                if (_buildingMinimumCapacity == value) return;
                _buildingMinimumCapacity = value;
                _model.SetMinimumCapacity(_selectedBuilding.Id, value);

                OnPropertyChanged("MinimumCapacity");
            }
        }

        public Int32 BuildingAdrenalin
        {
            get { return _buildingAdrenalin; }
            set
            {
                if (_buildingAdrenalin == value) return;
                _buildingAdrenalin = value;

                BuildingPopupAdrenalin = "Adrenalin: " + _buildingAdrenalin.ToString();
                OnPropertyChanged("BuildingPopupAdrenalin");
                OnPropertyChanged("BuildingAdrenalin");
            }
        }

        public String BuildingPopupAdrenalin { get; set; }

        public String BuildingPopupCapacity { get; set; }

        public String BuildingPopupQueue { get; set; }

        public String BuildingPopupStatus { get; set; }

        public Int32 BuildingCurrentCapacity { get; set; }

        public Int32 BuildingMaxCapacity { get; set; }

        public PersonField VisitorPopupSource { get; set; }

        public Boolean VisitorInfoPopup
        {
            get { return _visitorInfoPopup; }
            set
            {
                if (_visitorInfoPopup == value) return;
                _visitorInfoPopup = value;
                OnPropertyChanged("VisitorInfoPopup");
            }
        }

        public Int32 VisitorInfoX
        {
            get { return _visitorInfoX; }
            set
            {
                if (_visitorInfoX == value) return;
                _visitorInfoX = value;
                OnPropertyChanged("VisitorInfoX");
            }
        }

        public Int32 VisitorInfoY
        {
            get { return _visitorInfoY; }
            set
            {
                if (_visitorInfoY == value) return;
                _visitorInfoY = value;
                OnPropertyChanged("VisitorInfoY");
            }
        }

        public Status VisitorStatus
        {
            get { return _visitorStatus; }
            set
            {
                if (_visitorStatus == value) return;
                _visitorStatus = value;

                VisitorPopupStatus = "Status: " + value.ToString().ToLower();
                OnPropertyChanged("VisitorPopupStatus");
            }
        }

        public String VisitorPopupStatus { get; set; }

        public MapData.People.Activity VisitorActivity
        {
            get { return _visitorActivity; }
            set
            {
                if (value == _visitorActivity) return;
                _visitorActivity = value;

                VisitorPoupActivity = "Activity: " + value.ToString().ToLower().Replace('_',' ');
                OnPropertyChanged("VisitorPoupActivity");
            }
        }

        public String VisitorPoupActivity { get; set; }

        public Int32 VisitorMoney
        {
            get { return _visitorMoney; }
            set
            {
                if (_visitorMoney == value) return;
                _visitorMoney = value;

                VisitorPopupMoney = "Money: " + _visitorMoney.ToString();
                OnPropertyChanged("VisitorPopupMoney");
            }
        }

        public String VisitorPopupMoney { get; set; }

        public Int32 VisitorFood
        {
            get { return _visitorFood; }
            set
            {
                if (_visitorFood == value) return;
                _visitorFood = value;

                VisitorPopupFood = "Food: " + _visitorFood.ToString();
                OnPropertyChanged("VisitorFood");
                OnPropertyChanged("VisitorPopupFood");
            }
        }

        public String VisitorPopupFood { get; set; }

        public Int32 VisitorAdrenalin
        {
            get { return _visitorAdrenalin; }
            set
            {
                if (_visitorAdrenalin == value) return;
                _visitorAdrenalin = value;

                VisitorPopupAdrenalin = "Adrenalin: " + _visitorAdrenalin.ToString();
                OnPropertyChanged("VisitorPopupAdrenalin");
                OnPropertyChanged("VisitorAdrenalin");
            }
        }

        public String VisitorPopupAdrenalin { get; set; }

        public Int32 VisitorHappiness
        {
            get { return _visitorHappiness; }
            set
            {
                if (_visitorHappiness == value) return;
                _visitorHappiness = value;

                VisitorPopupHappiness = "Happiness: " + _visitorHappiness.ToString();
                OnPropertyChanged("VisitorPopupHappiness");
                OnPropertyChanged("VisitorHappiness");
            }
        }

        public String VisitorPopupHappiness { get; set; }

        public String ParkName
        {
            get { return _parkName; }
            set { _parkName = value; OnPropertyChanged("ParkName"); }
        }

        public Int32 ParkMoney
        {
            get { return _parkMoney; }
            set { _parkMoney = value; OnPropertyChanged("ParkMoney"); }
        }

        public Int32 ParkGuests
        {
            get { return _parkGuests; }
            set { _parkGuests = value; OnPropertyChanged("ParkGuests"); }
        }

        public GameMode CurrentGameMode { get; set; }

        public Int32 Income
        {
            get { return _income; }
            set { _income = value; OnPropertyChanged("Income"); }
        }

        public String Time
        {
            get { return _time; }
            set { _time = value; OnPropertyChanged("Time"); }
        }

        public Double Zoom
        {
            get { return _zoom; }
            set
            {
                if (_zoom == value) return;
                if (value < 1)
                {
                    _zoom = 1;
                }
                else if (value > 3)
                {
                    _zoom = 3;
                }
                else
                {
                    _zoom = value;
                }

                OnPropertyChanged("Zoom");
            }
        }

        #endregion

        #region Observable Collections

        public ObservableCollection<PersonField> Customers { get; private set; }
        public ObservableCollection<FacilityField> Facilities { get; private set; }
        public ObservableCollection<Field> Roads { get; private set; }
        public ObservableCollection<Field> Fields { get; private set; }
        public ObservableCollection<MenuField> MenuItems { get; private set; }

        #endregion

        #region Commands

        public DelegateCommand ClickOnCanvas { get; private set; }
        public DelegateCommand ZoomOnCanvas { get; private set; }
        public DelegateCommand BuildCommand { get; private set; }
        public DelegateCommand DestroyCommand { get; private set; }
        public DelegateCommand ExitGameCommand { get; private set; }
        public DelegateCommand GetBuildingInfoCommand { get; private set; }
        public DelegateCommand GetVisitorInfoCommand { get; private set; }
        public DelegateCommand HireServiceCommand { get; private set; }
        public DelegateCommand TitleScreenCommand { get; private set; }
        public DelegateCommand OpenParkCommand { get; private set; }
        public DelegateCommand SaveGameCommand { get; private set; }
        public DelegateCommand SetEntryFeeCommand { get; private set; }
        public DelegateCommand SlowDownTimeCommand { get; private set; }
        public DelegateCommand SpeedUpTimeCommand { get; private set; }
        public DelegateCommand StopTimeCommand { get; private set; }
        public DelegateCommand SettingsCommand { get; private set; }

        #endregion

        #region Constructors

        public GameViewModel(GameModel model)
        {
            _model = model;

            _model.BuildingChanged += new EventHandler<BuildingChangedEventArgs>(Model_BuildingChanged);
            _model.BuildingAdded += new EventHandler<BuildingChangedEventArgs>(Model_BuildingAdded);
            _model.BuildingDestroyed += new EventHandler<BuildingChangedEventArgs>(Model_BuildingDestroyed);
            _model.VisitorsChanged += new EventHandler(Model_VisitorsChanged);
            _model.PersonChanged += new EventHandler<PersonChangedEventArgs>(Model_PersonChanged);
            _model.PersonAdded += new EventHandler<PersonChangedEventArgs>(Model_PersonAdded);
            _model.PersonDeleted += new EventHandler<PersonChangedEventArgs>(Model_PersonDeleted);
            _model.NewGameStarted += new EventHandler<NewGameEventArgs>(Model_NewGameStarted);
            _model.SpeedChanged += new EventHandler<SpeedChangedEventArgs>(Model_SpeedChanged);
            _model.TimePassed += new EventHandler(Model_TimePassed);
            _model.MoneyChanged += new EventHandler(Model_MoneyChanged);
            _model.EnableDestroy += new EventHandler(Model_EnableDestroy);
            _model.PopupRefreshed += new EventHandler(Model_PopupRefreshed);

            _selectedBuilding = null;
            _visitorActivity = MapData.People.Activity.USING;
            _visitorStatus = MapData.People.Status.UNHAPPY;
            _lastSpeed = Speed.NORMAL;
            SelectedItem = new MenuField();
            Zoom = 1.0;
            Field.ImageSize = 50;
            CurrentGameMode = GameMode.NORMAL;
            _mapSize = GameModel.MapSize;
            CanvasWidth = _mapSize * Field.ImageSize;
            CanvasHeight = _mapSize * Field.ImageSize;
            PanelWidth = 125;
            WindowWidth = PanelWidth + CanvasWidth + 44;
            WindowHeight = 40 + CanvasHeight + 43;
            BackgroundImage = new Uri("resources/BaseTerrain.png", UriKind.Relative);
            BuildingInfoPopup = false;
            VisitorInfoPopup = false;
            ErrorY = 40 + CanvasHeight + 43 - 110;
            ErrorX = PanelWidth + CanvasWidth + 40 - 190;
            BuildingInfoX = VisitorInfoX = PanelWidth + 40;
            BuildingInfoY = VisitorInfoY = CanvasHeight - 120;


            ClickOnCanvas = new DelegateCommand(param => CanvasClicked(param));
            ZoomOnCanvas = new DelegateCommand(param => CanvasZoomed((String)param));
            BuildCommand = new DelegateCommand(param => BuildClicked());
            DestroyCommand = new DelegateCommand(param => DestroyClicked());
            SlowDownTimeCommand = new DelegateCommand(param => ChangeTimeSpeed(-1));
            SpeedUpTimeCommand = new DelegateCommand(param => ChangeTimeSpeed(1));
            StopTimeCommand = new DelegateCommand(param => ChangeTimeSpeed(0));
            TitleScreenCommand = new DelegateCommand(param => OnBackToTitleScreen());
            SettingsCommand = new DelegateCommand(param => OnOpenSettings());
            SaveGameCommand = new DelegateCommand(param => OnSaveGame());
            OpenParkCommand = new DelegateCommand(param => ChangeParkStatus());

            Fields = new ObservableCollection<Field>();
            MenuItems = new ObservableCollection<MenuField>();

            MenuItems.Add(new MenuField { Name = "Road", Price = _model.BuildingData.RoadData.Price, TypeId = _model.BuildingData.RoadData.BuildingTypeId, ImageSource = new Uri("resources/Road.png", UriKind.Relative), SelectCommand = new DelegateCommand(item => ItemClicked(item as MenuField)), ExtraInfo = String.Empty });

            foreach (CateringData building in _model.BuildingData.CateringData)
            {
                MenuItems.Add(new MenuField { Name = System.Text.RegularExpressions.Regex.Replace(building.Name, "([A-Z])", " $1", System.Text.RegularExpressions.RegexOptions.Compiled).Trim(), Price = building.Price, TypeId = building.BuildingTypeId, ImageSource = new Uri("resources/" + building.Name + ".png", UriKind.Relative), SelectCommand = new DelegateCommand(item => ItemClicked(item as MenuField)), ExtraInfo = "Size: " + building.Size.Item1 + " x " + building.Size.Item2 + "\nCapacity: " + building.Capacity + "\nFood: " + building.Food });
            }
            foreach (EntertainmentData building in _model.BuildingData.EntertainmentData)
            {
                MenuItems.Add(new MenuField { Name = System.Text.RegularExpressions.Regex.Replace(building.Name, "([A-Z])", " $1", System.Text.RegularExpressions.RegexOptions.Compiled).Trim(), Price = building.Price, TypeId = building.BuildingTypeId, ImageSource = new Uri("resources/" + building.Name + ".png", UriKind.Relative), SelectCommand = new DelegateCommand(item => ItemClicked(item as MenuField)), ExtraInfo = "Size: " + building.Size.Item1 + " x " + building.Size.Item2 + "\nCapacity: " + building.Capacity + "\nHappiness: " + building.Happiness + "\nAdrenalin: " + building.Adrenalin });
            }
        }

        public void UnSubscribeFromModelEvents()
        {
            _model.BuildingChanged -= Model_BuildingChanged;
            _model.BuildingAdded -= Model_BuildingAdded;
            _model.BuildingDestroyed -= Model_BuildingDestroyed;
            _model.VisitorsChanged -= Model_VisitorsChanged;
            _model.PersonChanged -= Model_PersonChanged;
            _model.PersonAdded -= Model_PersonAdded;
            _model.PersonDeleted -= Model_PersonDeleted;
            _model.NewGameStarted -= Model_NewGameStarted;
            _model.SpeedChanged -= Model_SpeedChanged;
            _model.TimePassed -= Model_TimePassed;
            _model.MoneyChanged -= Model_MoneyChanged;
            _model.EnableDestroy -= Model_EnableDestroy;
            _model.PopupRefreshed -= Model_PopupRefreshed;
        }

        #endregion

        #region Events

        public event EventHandler ExitGame;
        public event EventHandler BackToTitleScreen;
        public event EventHandler OpenSettings;
        public event EventHandler SaveGame;

        #endregion

        #region Private methods

        private void ChangeParkStatus()
        {
            if(_model.ParkStatus == ParkStatus.CLOSED)
            {
                _model.ParkStatus = ParkStatus.OPEN;
                CurrentGameMode = GameMode.NORMAL;
                OpenParkText = "Close";
                DestroyVisibility = System.Windows.Visibility.Collapsed;
                TimeVisibility = System.Windows.Visibility.Visible;
                ChangeTimeSpeed(0);
            }
            else
            {
                _model.ParkStatus = ParkStatus.CLOSED;
                OpenParkText = "Open";
                TimeVisibility = System.Windows.Visibility.Collapsed;
                OpenVisibility = System.Windows.Visibility.Collapsed;
            }
        }

        private void ChangeTimeSpeed(Int32 mod)
        {
            if (mod == 0)
            {
                if (_model.GameSpeed == Speed.PAUSE)
                {
                    PauseText = "Pause";
                    _model.GameSpeed = _lastSpeed;
                } else
                {
                    PauseText = "Resume";
                    _lastSpeed = _model.GameSpeed;
                    _model.GameSpeed = Speed.PAUSE;
                }
            }
            else if (mod == 1 && _model.GameSpeed != Speed.FAST && _model.GameSpeed != Speed.PAUSE)
            {
                _model.GameSpeed = _model.GameSpeed + 1;
            }
            else if (mod == -1 && _model.GameSpeed != Speed.SLOW && _model.GameSpeed != Speed.PAUSE)
            {
                _model.GameSpeed = _model.GameSpeed - 1;
            }
            SpeedText = _model.GameSpeed.ToString();
        }

        private void ShowErrorPopup(String message)
        {
            ErrorMessage = message;
            ErrorPopup = true;

            DispatcherTimer timer = new DispatcherTimer()
            {
                Interval = TimeSpan.FromSeconds(5)
            };

            timer.Tick += delegate (object sender, EventArgs e)
            {
                ((DispatcherTimer)timer).Stop();
                if (ErrorPopup) ErrorPopup = false;
            };

            timer.Start();
        }

        private void ItemClicked(MenuField item)
        {
            if (IsSelected)
            {
                if (item == SelectedItem)
                {
                    item.Background = Brushes.Transparent;
                    IsSelected = false;
                }
                else
                {
                    SelectedItem.Background = Brushes.Transparent;
                    item.Background = Brushes.Red;
                    IsSelected = true;
                    SelectedItem = item;
                }
            }
            else
            {
                SelectedItem.Background = Brushes.Transparent;
                item.Background = Brushes.Red;
                IsSelected = true;
                SelectedItem = item;
            }
        }

        private void BuildClicked()
        {
            switch (CurrentGameMode)
            {
                case GameMode.NORMAL:
                    BuildBackground = Brushes.Green;
                    CurrentGameMode = GameMode.BUILD;
                    AddLines();
                    break;

                case GameMode.BUILD:
                    BuildBackground = (SolidColorBrush)new BrushConverter().ConvertFrom("#4A1919");
                    CurrentGameMode = GameMode.NORMAL;
                    DeleteLines();
                    break;

                case GameMode.DESTROY:
                    BuildBackground = Brushes.Green;
                    DestroyBackground = (SolidColorBrush)new BrushConverter().ConvertFrom("#4A1919");
                    CurrentGameMode = GameMode.BUILD;
                    AddLines();
                    break;
            }
        }

        private void DestroyClicked()
        {
            switch (CurrentGameMode)
            {
                case GameMode.NORMAL:
                    if(_model.ParkStatus != ParkStatus.OPEN)
                    {
                        DestroyBackground = Brushes.Red;
                        CurrentGameMode = GameMode.DESTROY;
                    }
                    break;

                case GameMode.BUILD:
                    if (_model.ParkStatus != ParkStatus.OPEN)
                    {
                        DestroyBackground = Brushes.Red;
                        BuildBackground = (SolidColorBrush)new BrushConverter().ConvertFrom("#4A1919");
                        CurrentGameMode = GameMode.DESTROY;
                        DeleteLines();
                    }
                    break;

                case GameMode.DESTROY:
                    DestroyBackground = (SolidColorBrush)new BrushConverter().ConvertFrom("#4A1919");
                    CurrentGameMode = GameMode.NORMAL;
                    break;
            }
        }

        private void AddLines()
        { 
            for(int i = 1; i < _mapSize; i++)
            {
                Fields.Add(new Fields.Line { Z = -1, Size = new Tuple<int, int>(0, _mapSize), Coord = new Tuple<int, int>(i, 0), ImageSource = new Uri("resources/lineh.png", UriKind.Relative) });
                Fields.Add(new Fields.Line { Z = -1, Size = new Tuple<int, int>(_mapSize, 0), Coord = new Tuple<int, int>(0, i), ImageSource = new Uri("resources/linev.png", UriKind.Relative) });
            }
        }

        private void DeleteLines()
        {
            foreach(Field field in Fields.ToList())
            {
                if(field is Fields.Line)
                {
                    Fields.Remove(field);
                }
            }
        }

        private void CanvasZoomed(String param)
        {
            if(param == "-")
            {
                Zoom -= 0.1;
            }
            else if(param == "+")
            {
                Zoom += 0.05;
            }
        }

        private void CanvasClicked(object arg)
        {
            MouseEventArgs e = (MouseEventArgs)arg;
            var position = e.GetPosition(e.Device.Target);

            switch(CurrentGameMode)
            {
                case GameMode.NORMAL:
                    if (e.OriginalSource is Image)
                    {
                        Image rec = (Image)e.OriginalSource;
                        Field field = (Field)rec.DataContext;

                        if(field is FacilityField)
                        {
                            FacilityField facility = field as FacilityField;
                            BuildingPopupSource = facility;
                            BuildingInfo info = _model.GetBuildingInfo(facility.Id);
                            _selectedBuilding = facility;
                            BuildingPopupName = info.Name;
                            Status = info.Status;
                            Fee = info.Fee;
                            Capacity = info.Capacity;
                            if(info is EntertainmentInfo)
                            {
                                MinimumCapacity = (info as EntertainmentInfo).MinimumCapacity;
                                BuildingAdrenalin = (info as EntertainmentInfo).Adrenalin;
                                MinimumCapacityVisibility = System.Windows.Visibility.Visible;
                                BuildingAdrenalinVisibility = System.Windows.Visibility.Visible;
                            }
                            else
                            {
                                MinimumCapacityVisibility = System.Windows.Visibility.Collapsed;
                                BuildingAdrenalinVisibility = System.Windows.Visibility.Collapsed;
                            }

                            BuildingInfoPopup = true;
                            VisitorInfoPopup = false;
                            _model.PopupIsActive = true;
                        }
                        else if(field is PersonField)
                        {
                            PersonField visitor = field as PersonField;
                            VisitorPopupSource = visitor;
                            VisitorInfo info = _model.GetVisitorInfo(visitor.Id);
                            if (info != null)
                            {
                                VisitorMoney = info.Money;
                                VisitorFood = info.Food;
                                VisitorAdrenalin = info.Adrenalin;
                                VisitorHappiness = info.Happiness;
                                VisitorActivity = info.CurrentActivity;
                                VisitorStatus = info.CurrentStatus;

                                VisitorInfoPopup = true;
                                BuildingInfoPopup = false;
                                _model.PopupIsActive = true;
                            }
                        }
                    } else
                    {
                        BuildingInfoPopup = false;
                        VisitorInfoPopup = false;
                        _model.PopupIsActive = false;
                    }
                    break;
                case GameMode.BUILD:
                    if (IsSelected)
                    {
                        if (!(e.OriginalSource is Image))
                        {
                            try
                            {
                                _model.Build(new Tuple<int, int>((int)position.Y / 50, (int)position.X / 50), SelectedItem.TypeId);
                            } 
                            catch(Model.ModelExceptions.BuildException ex)
                            {
                                switch(ex.ErrorType)
                                {
                                    case Model.ModelExceptions.BuildErrorType.NOT_ENOUGH_MONEY:
                                        ShowErrorPopup("You don't have enough money!");
                                        break;
                                    case Model.ModelExceptions.BuildErrorType.OFF_MAP:
                                        ShowErrorPopup("You can't build there");
                                        break;
                                    case Model.ModelExceptions.BuildErrorType.SPACE_OCCUPIED:
                                        ShowErrorPopup("Land is occupied!");
                                        break;
                                }
                            }
                            
                        }
                    }
                    break;
                case GameMode.DESTROY:
                    if(e.OriginalSource is Image)
                    {
                        Image toBeDestroyed = (Image)e.OriginalSource;
                        foreach(Field field in Fields)
                        {
                            if(field == toBeDestroyed.DataContext)
                            {
                                if(!(field is GateField))
                                {
                                    _model.Destroy(field.Coord, field.Size);
                                    break;
                                }
                            }
                        }
                    }
                    break;
            }
        }

        private void Model_BuildingAdded(Object o, BuildingChangedEventArgs e)
        {
            String uri;
            FacilityData facility;
            StaticBuildingData roadOrGate;

            if (e.Building is Gate)
            {
                roadOrGate = _model.BuildingData.GetBuilding(e.Building.BuildingTypeId);
                Fields.Add(new GateField { Z = 0, Size = roadOrGate.Size, Coord = e.Building.Coord, ImageSource = new Uri("resources/Gate.png", UriKind.Relative) });
            }
            else if (e.Building is Road)
            {
                uri = SelectRoad(e.Building.Coord);
                roadOrGate = _model.BuildingData.GetBuilding(e.Building.BuildingTypeId);
                Fields.Add(new RoadField { Z = 0, Size = roadOrGate.Size, Coord = e.Building.Coord, ImageSource = new Uri(uri, UriKind.Relative), Id = e.Building.BuildingId });
                var Roads = Fields.Where(x => x is RoadField);
                var Road = Roads.Single(x => ((RoadField)x).Coord.Equals(e.Building.Coord));
                var NeighbourRoads = Roads.Where(x => ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1 - 1, e.Building.Coord.Item2)) ||
                                                       ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1 + 1, e.Building.Coord.Item2)) ||
                                                       ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1, e.Building.Coord.Item2 - 1)) ||
                                                       ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1, e.Building.Coord.Item2 + 1)));
                foreach (Field road in NeighbourRoads)
                {
                    uri = SelectRoad(road.Coord);
                    road.ImageSource = new Uri(uri, UriKind.Relative);
                }
                CollectionViewSource.GetDefaultView(Fields).Refresh();
            }
            else
            {
                facility = (FacilityData)_model.BuildingData.GetBuilding(e.Building.BuildingTypeId);
                Fields.Add(new FacilityField { Z = 0, Size = facility.Size, Coord = e.Building.Coord, ImageSource = new Uri("resources/UCG.gif", UriKind.Relative), Id = e.Building.BuildingId });
                var Roads = Fields.Where(x => x is RoadField);
                foreach (Field road in Roads)
                {
                    uri = SelectRoad(road.Coord);
                    road.ImageSource = new Uri(uri, UriKind.Relative);
                }
                CollectionViewSource.GetDefaultView(Fields).Refresh();
            }
        }

        private void Model_BuildingDestroyed(Object o, BuildingChangedEventArgs e)
        {
            if(e.Building.BuildingTypeId == 0)
            {
                var Roads = Fields.Where(x => x is RoadField);
                var Road = Roads.Single(x => ((RoadField)x).Coord.Equals(e.Building.Coord));
                var NeighbourRoads = Roads.Where(x => ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1 - 1, e.Building.Coord.Item2)) ||
                                                       ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1 + 1, e.Building.Coord.Item2)) ||
                                                       ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1, e.Building.Coord.Item2 - 1)) ||
                                                       ((RoadField)x).Coord.Equals(new Tuple<Int32, Int32>(e.Building.Coord.Item1, e.Building.Coord.Item2 + 1)));
                String uri;
                foreach(Field road in NeighbourRoads)
                {
                    uri = SelectRoad(road.Coord);
                    road.ImageSource = new Uri(uri, UriKind.Relative);
                }
                Fields.Remove(Road);
                CollectionViewSource.GetDefaultView(Fields).Refresh();
            }
            else
            {
                var Facs = Fields.Where(x => x is FacilityField);
                var Fac = Facs.Single(x => ((FacilityField)x).Coord.Equals(e.Building.Coord));
                Fields.Remove(Fac);
            }
        }

        private void Model_BuildingChanged(Object o, BuildingChangedEventArgs e)
        {
            var Facs = Fields.Where(x => x is FacilityField);
            var Fac = Facs.Single(x => ((FacilityField)x).Id == e.Building.BuildingId);
            var f = (FacilityData)_model.BuildingData.GetBuilding(e.Building.BuildingTypeId);
            Fac.ImageSource = new Uri("resources/" + f.Name + ".png", UriKind.Relative);
        }

        private void Model_PersonChanged(Object o, PersonChangedEventArgs e)
        {
            var Visitors = Fields.Where(x => x is PersonField);
            var visitor = Visitors.Single(x => ((PersonField)x).Id == e.Person.PersonId);
            ((PersonField)visitor).InTileCoord = e.Person.InTileCoord;
            ((PersonField)visitor).Coord = e.Person.Coord;
        }

        private void Model_PersonDeleted(Object o, PersonChangedEventArgs e)
        {
            var Visitors = Fields.Where(x => x is PersonField);
            var visitor = Visitors.Single(x => ((PersonField)x).Id == e.Person.PersonId);
            Fields.Remove(visitor);
        }

        private void Model_PersonAdded(Object o, PersonChangedEventArgs e)
        {
            if (e.Person is Visitor)
            {
                Fields.Add(new PersonField { Z = 1, Id = e.Person.PersonId, Size = new Tuple<int, int>(1, 1), InTileCoord = e.Person.InTileCoord, Coord = e.Person.Coord, ImageSource = new Uri("resources/Ghost.png", UriKind.Relative) });
            }
            else if (e.Person is Repairman)
            {
                Fields.Add(new PersonField { Z = 1, Id = e.Person.PersonId, Size = new Tuple<int, int>(1, 1), InTileCoord = e.Person.InTileCoord, Coord = e.Person.Coord, ImageSource = new Uri("resources/Repairman.png", UriKind.Relative) });
            }
        }

        private void Model_VisitorsChanged(Object o, EventArgs e)
        {
            ParkGuests = _model.GameMap.Visitors.Count;
            ParkMoney = _model.GameMap.Money;
            CollectionViewSource.GetDefaultView(Fields).Refresh();
        }

        private String SelectRoad(Tuple<Int32, Int32> coord)
        {
            String name = String.Empty;
            Int32 X = coord.Item2;
            Int32 Y = coord.Item1;
            Int32 Edge = _mapSize - 1;

            if (X == 0 || Y == 0 || X == Edge || Y == Edge)
            {
                if (X == 0 && Y == 0)
                {
                    if (_model[X, Y + 1] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "CurvedBottomRight";
                    }
                    else if (_model[X + 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else
                    {
                        name = "LineBottomTop";
                    }
                }
                else if (X == Edge && Y == Edge)
                {
                    if (_model[X, Y - 1] != -1 && _model[X - 1, Y] != -1)
                    {
                        name = "CurvedLeftTop";
                    }
                    else if (_model[X - 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else
                    {
                        name = "LineBottomTop";
                    }
                }
                else if (X == 0 && Y == Edge)
                {
                    if (_model[X, Y - 1] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "CurvedTopRight";
                    }
                    else if (_model[X + 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else
                    {
                        name = "LineBottomTop";
                    }
                }
                else if (X == Edge && Y == 0)
                {
                    if (_model[X, Y + 1] != -1 && _model[X - 1, Y] != -1)
                    {
                        name = "CurvedLeftBottom";
                    }
                    else if (_model[X - 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else
                    {
                        name = "LineBottomTop";
                    }
                }
                else if (X == 0)
                {
                    if (_model[X, Y + 1] != -1 && _model[X, Y - 1] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "ThreeWayRight";
                    }
                    else if (_model[X, Y - 1] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "CurvedTopRight";
                    }
                    else if (_model[X, Y + 1] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "CurvedBottomRight";
                    }
                    else if (_model[X + 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else
                    {
                        name = "LineBottomTop";
                    }
                }
                else if (Y == 0)
                {
                    if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1 && _model[X, Y + 1] != -1)
                    {
                        name = "ThreeWayBottom";
                    }
                    else if (_model[X - 1, Y] != -1 && _model[X, Y + 1] != -1)
                    {
                        name = "CurvedLeftBottom";
                    }
                    else if (_model[X + 1, Y] != -1 && _model[X, Y + 1] != -1)
                    {
                        name = "CurvedBottomRight";
                    }
                    else if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else if (_model[X, Y + 1] != -1)
                    {
                        name = "LineBottomTop";
                    }
                    else
                    {
                        name = "LineLeftRight";
                    }
                }
                else if (X == Edge)
                {
                    if (_model[X, Y - 1] != -1 && _model[X, Y + 1] != -1 && _model[X - 1, Y] != -1)
                    {
                        name = "ThreeWayLeft";
                    }
                    else if (_model[X, Y - 1] != -1 && _model[X - 1, Y] != -1)
                    {
                        name = "CurvedLeftTop";
                    }
                    else if (_model[X, Y + 1] != -1 && _model[X - 1, Y] != -1)
                    {
                        name = "CurvedLeftBottom";
                    }
                    else if (_model[X - 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else
                    {
                        name = "LineBottomTop";
                    }
                }
                else if (Y == Edge)
                {
                    if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1 && _model[X, Y - 1] != -1)
                    {
                        name = "ThreeWayTop";
                    }
                    else if (_model[X - 1, Y] != -1 && _model[X, Y - 1] != -1)
                    {
                        name = "CurvedLeftTop";
                    }
                    else if (_model[X + 1, Y] != -1 && _model[X, Y - 1] != -1)
                    {
                        name = "CurvedTopRight";
                    }
                    else if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1)
                    {
                        name = "LineLeftRight";
                    }
                    else if (_model[X, Y - 1] != -1)
                    {
                        name = "LineBottomTop";
                    }
                    else
                    {
                        name = "LineLeftRight";
                    }
                }
                else
                {
                    name = "LineBottomTop";
                }
            }
            else
            {
                if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1 && _model[X, Y - 1] != -1 && _model[X, Y + 1] != -1)
                {
                    name = "FourWay";
                }
                else if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1 && _model[X, Y - 1] != -1)
                {
                    name = "ThreeWayTop";
                }
                else if (_model[X - 1, Y] != -1 && _model[X + 1, Y] != -1 && _model[X, Y + 1] != -1)
                {
                    name = "ThreeWayBottom";
                }
                else if (_model[X - 1, Y] != -1 && _model[X, Y - 1] != -1 && _model[X, Y + 1] != -1)
                {
                    name = "ThreeWayLeft";
                }
                else if (_model[X + 1, Y] != -1 && _model[X, Y - 1] != -1 && _model[X, Y + 1] != -1)
                {
                    name = "ThreeWayRight";
                }
                else if (_model[X - 1, Y] != -1 && _model[X, Y - 1] != -1)
                {
                    name = "CurvedLeftTop";
                }
                else if (_model[X - 1, Y] != -1 && _model[X, Y + 1] != -1)
                {
                    name = "CurvedLeftBottom";
                }
                else if (_model[X + 1, Y] != -1 && _model[X, Y + 1] != -1)
                {
                    name = "CurvedBottomRight";
                }
                else if (_model[X + 1, Y] != -1 && _model[X, Y - 1] != -1)
                {
                    name = "CurvedTopRight";
                }
                else if (_model[X - 1, Y] != -1 || _model[X + 1, Y] != -1)
                {
                    name = "LineLeftRight";
                }
                else
                {
                    name = "LineBottomTop";
                }
            }
            return "resources/" + name + ".png";
        }

        private void Model_SpeedChanged(Object o, SpeedChangedEventArgs e)
        {
            if (_model.GameSpeed == Speed.PAUSE) { PauseText = "Resume"; }
            else PauseText = "Pause";
            SpeedText = _model.GameSpeed.ToString();
        }

        private void Model_NewGameStarted(Object o, NewGameEventArgs e)
        {
            if (_model.GameSpeed == Speed.PAUSE) { PauseText = "Resume"; }
            else PauseText = "Pause";
            if (_model.ParkStatus == ParkStatus.CLOSED) { OpenParkText = "Open"; }
            else OpenParkText = "Close";
            SpeedText = _model.GameSpeed.ToString();

            ParkName = e.Name;
            ParkMoney = e.GameMap.Money;
            ParkGuests = e.GameMap.Visitors.Count;
            Time = ("00:00:00");
            CurrentGameMode = GameMode.NORMAL;
            DestroyVisibility = System.Windows.Visibility.Visible;
            TimeVisibility = System.Windows.Visibility.Collapsed;
            OpenVisibility = System.Windows.Visibility.Visible;

        }

        private void Model_MoneyChanged(Object o, EventArgs e)
        {
            ParkMoney = _model.GameMap.Money;
        }

        private void Model_TimePassed(Object o, EventArgs e)
        {
            Time = TimeSpan.FromSeconds(_model.GameMap.Time).ToString(@"hh\:mm\:ss");
        }

        private void Model_EnableDestroy(Object o, EventArgs e)
        {
            DestroyVisibility = System.Windows.Visibility.Visible;
            OpenVisibility = System.Windows.Visibility.Visible;
        }

        private void Model_PopupRefreshed(Object o, EventArgs e)
        {
            if (VisitorInfoPopup)
            {
                VisitorInfo info = _model.GetVisitorInfo(VisitorPopupSource.Id);
                if (info != null)
                {
                    VisitorMoney = info.Money;
                    VisitorFood = info.Food;
                    VisitorAdrenalin = info.Adrenalin;
                    VisitorHappiness = info.Happiness;
                    VisitorActivity = info.CurrentActivity;
                    VisitorStatus = info.CurrentStatus;
                }
            }
            else if (BuildingInfoPopup)
            {
                BuildingInfo info = _model.GetBuildingInfo(BuildingPopupSource.Id);
                _selectedBuilding = BuildingPopupSource;
                BuildingPopupName = info.Name;
                Status = info.Status;
                Fee = info.Fee;
                Capacity = info.Capacity;
                if (info is EntertainmentInfo)
                {
                    MinimumCapacity = (info as EntertainmentInfo).MinimumCapacity;
                    BuildingAdrenalin = (info as EntertainmentInfo).Adrenalin;
                    MinimumCapacityVisibility = System.Windows.Visibility.Visible;
                    BuildingAdrenalinVisibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    MinimumCapacityVisibility = System.Windows.Visibility.Collapsed;
                    BuildingAdrenalinVisibility = System.Windows.Visibility.Collapsed;
                }
            }
        }
        #endregion

        #region Private event methods

        private void OnExitGame()
        {
            ExitGame?.Invoke(this, EventArgs.Empty);
        }

        private void OnBackToTitleScreen()
        {
            BackToTitleScreen?.Invoke(this, EventArgs.Empty);
        }

        private void OnOpenSettings()
        {
            OpenSettings?.Invoke(this, EventArgs.Empty);
        }

        private void OnSaveGame()
        {
            SaveGame?.Invoke(this, EventArgs.Empty);
        }

        #endregion
    }
}
